//********************* Base64 *******************************************
//* 
//* The Base64 class below was adapted from XML-RPC Client
//* because I didn't find a scriptable Mozilla interface 
//* that will do Base64 encoding/decoding
//*
//* XML-RPC Client is subject to the Mozilla Public License Version
//* 1.1 (the "License"); you may not use this file except in compliance with
//* the License. You may obtain a copy of the License at http://www.mozilla.org/MPL 
//* 
//* Software distributed under the License is distributed on an "AS IS" basis,
//* WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
//* for the specific language governing rights and limitations under the
//* License.
//*
//* The Original Code is Mozilla XML-RPC Client component.
//*
//* The Initial Developer of the Original Code is
//* Digital Creations 2, Inc.
//* Portions created by the Initial Developer are Copyright (C) 2000
//* the Initial Developer. All Rights Reserved.
//*
//* Contributor(s):
//*   Martijn Pieters <mj@digicool.com> (original author)
//*   Samuel Sieb <samuel@sieb.net> brought it up to date with
//*   current APIs and added authentication
//*
//*

/* START BASE64 CODE */
var TwittyTunesBase64 = {

    base64Pad : '=',

    toBase64Table : 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz' +
    '0123456789+/',

    toBinaryTable : [
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,-1,
        -1,-1,-1,-1, -1,-1,-1,-1, -1,-1,-1,62, -1,-1,-1,63,
        52,53,54,55, 56,57,58,59, 60,61,-1,-1, -1, 0,-1,-1,
        -1, 0, 1, 2,  3, 4, 5, 6,  7, 8, 9,10, 11,12,13,14,
        15,16,17,18, 19,20,21,22, 23,24,25,-1, -1,-1,-1,-1,
        -1,26,27,28, 29,30,31,32, 33,34,35,36, 37,38,39,40,
        41,42,43,44, 45,46,47,48, 49,50,51,-1, -1,-1,-1,-1
    ],

    encode : function(string) {
        var data = []
        for (var i=0; i < string.length; i++) {
            var d = string.charCodeAt(i);
            data = data.concat(isNaN(d) ? 0 : d);
        }
        return this.toBase64(data);
    },

    decode : function(data) {
        var result = '';
        var leftbits = 0; // number of bits decoded, but yet to be appended
        var leftdata = 0; // bits decoded, bt yet to be appended

        // Convert one by one.
        for (var i in data) {
            var c = this.toBinaryTable[data.charCodeAt(i) & 0x7f];
            var padding = (data[i] == this.base64Pad);
            // Skip illegal characters and whitespace
            if (c == -1) continue;
            
            // Collect data into leftdata, update bitcount
            leftdata = (leftdata << 6) | c;
            leftbits += 6;

            // If we have 8 or more bits, append 8 bits to the result
            if (leftbits >= 8) {
                leftbits -= 8;
                // Append if not padding.
                if (!padding)
                    result += String.fromCharCode((leftdata >> leftbits) & 0xff);
                leftdata &= (1 << leftbits) - 1;
            }
        }

        // If there are any bits left, the base64 string was corrupted
        if (leftbits)
            throw Components.Exception('Corrupted base64 string');

        return result;
    },

    toBase64 : function(data) {
        var result = '';
        var length = data.length;
        var i;
        // Convert every three bytes to 4 ascii characters.
        for (i = 0; i < (length - 2); i += 3) {
            result += this.toBase64Table[data[i] >> 2];
            result += this.toBase64Table[((data[i] & 0x03) << 4) + (data[i+1] >> 4)];
            result += this.toBase64Table[((data[i+1] & 0x0f) << 2) + (data[i+2] >> 6)];
            result += this.toBase64Table[data[i+2] & 0x3f];
        }

        // Convert the remaining 1 or 2 bytes, pad out to 4 characters.
        if (length%3) {
            i = length - (length%3);
            result += this.toBase64Table[data[i] >> 2];
            if ((length%3) == 2) {
                result += this.toBase64Table[((data[i] & 0x03) << 4) + (data[i+1] >> 4)];
                result += this.toBase64Table[(data[i+1] & 0x0f) << 2];
                result += this.base64Pad;
            } else {
                result += this.toBase64Table[(data[i] & 0x03) << 4];
                result += this.base64Pad + this.base64Pad;
            }
        }

        return result;
    }
}
